# -*- coding: utf-8 -*-

UNKNOWN = 0
INVALID_TAR = 1
ACCESS_DENIED = 2
NO_PACKAGE = 3
NO_LICENSE = 4
NO_CONFIG = 5
INVALID_CONFIG = 6
MISSING_TEMPLATE = 7
NAME_MISMATCH = 8
VERSION_TOO_LOW = 9
PRIVATE_BOILERPLATE_QUOTA = PRIVATE_APP_QUOTA = 10
INVALID_WEBSITE = 11
INVALID_FILENAME = 12
NO_WEBSITE = 13
MISSING_STATIC = 14
